@@ -24,7 +24,7 @@ using namespace log4cxx;
 using namespace log4cxx::spi;
 using namespace log4cxx::helpers;
 
-RootLogger::RootLogger(Pool& pool, const LevelPtr& level1) :
+RootLogger::RootLogger(Pool& pool, const LevelPtr level1) :
 	Logger(pool, LOG4CXX_STR("root"))
 {
 	setLevel(level1);
@@ -32,10 +32,10 @@ RootLogger::RootLogger(Pool& pool, const LevelPtr& level1) :
 
 const LevelPtr& RootLogger::getEffectiveLevel() const
 {
-	return level;
+	return getLevel();
 }
 
-void RootLogger::setLevel(const LevelPtr& level1)
+void RootLogger::setLevel(const LevelPtr level1)
 {
 	if (level1 == 0)
 	{
@@ -43,8 +43,7 @@ void RootLogger::setLevel(const LevelPtr& level1)
 	}
 	else
 	{
-
-		this->level = level1;
+		Logger::setLevel(level1);
 	}
 }
 
