@@ -19,12 +19,7 @@
 #define _LOG4CXX_HELPERS_ONLY_ONCE_ERROR_HANDLER_H
 
 #include <log4cxx/spi/errorhandler.h>
-#include <log4cxx/helpers/objectimpl.h>
-
-#ifdef _MSC_VER
-	#pragma warning ( push )
-	#pragma warning (disable : 4251) // ::std::exception needs to have dll-interface
-#endif
+#include <log4cxx/helpers/object.h>
 
 namespace log4cxx
 {
@@ -42,12 +37,10 @@ from being flooded with error messages when logging fails
 */
 class LOG4CXX_EXPORT OnlyOnceErrorHandler :
 	public virtual spi::ErrorHandler,
-	public virtual ObjectImpl
+	public virtual Object
 {
 	private:
-		LogString WARN_PREFIX;
-		LogString ERROR_PREFIX;
-		mutable bool firstTime;
+		LOG4CXX_DECLARE_PRIVATE_MEMBER_PTR(OnlyOnceErrorHandlerPrivate, m_priv)
 
 	public:
 		DECLARE_LOG4CXX_OBJECT(OnlyOnceErrorHandler)
@@ -57,56 +50,52 @@ class LOG4CXX_EXPORT OnlyOnceErrorHandler :
 		END_LOG4CXX_CAST_MAP()
 
 		OnlyOnceErrorHandler();
-		void addRef() const;
-		void releaseRef() const;
+
+		~OnlyOnceErrorHandler();
 
 		/**
 		 Does not do anything.
 		 */
-		void setLogger(const LoggerPtr& logger);
+		void setLogger(const LoggerPtr& logger) override;
 
 
 		/**
 		No options to activate.
 		*/
-		void activateOptions(log4cxx::helpers::Pool& p);
-		void setOption(const LogString& option, const LogString& value);
+		void activateOptions(helpers::Pool& p) override;
+		void setOption(const LogString& option, const LogString& value) override;
 
 
 		/**
 		Prints the message and the stack trace of the exception on
 		<code>System.err</code>.  */
 		void error(const LogString& message, const std::exception& e,
-			int errorCode) const;
+			int errorCode) const override;
 		/**
 		Prints the message and the stack trace of the exception on
 		<code>System.err</code>.
 		*/
 		void error(const LogString& message, const std::exception& e,
-			int errorCode, const spi::LoggingEventPtr& event) const;
+			int errorCode, const spi::LoggingEventPtr& event) const override;
 
 		/**
 		Print a the error message passed as parameter on
 		<code>System.err</code>.
 		*/
-		void error(const LogString& message) const;
+		void error(const LogString& message) const override;
 
 		/**
 		Does not do anything.
 		*/
-		void setAppender(const AppenderPtr& appender);
+		void setAppender(const AppenderPtr& appender) override;
 
 		/**
 		Does not do anything.
 		*/
-		void setBackupAppender(const AppenderPtr& appender);
+		void setBackupAppender(const AppenderPtr& appender) override;
 };
 }  // namespace helpers
 } // namespace log4cxx
 
-#if defined(_MSC_VER)
-	#pragma warning (pop)
-#endif
-
 #endif //_LOG4CXX_HELPERS_ONLY_ONCE_ERROR_HANDLER_H
 
