@@ -1,6 +1,6 @@
 /* poppler-form.h: qt interface to poppler
  * Copyright (C) 2007-2008, 2011, Pino Toscano <pino@kde.org>
- * Copyright (C) 2008, 2011, 2012, 2015-2020 Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2008, 2011, 2012, 2015-2023 Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2011 Carlos Garcia Campos <carlosgc@gnome.org>
  * Copyright (C) 2012, Adam Reichold <adamreichold@myopera.com>
  * Copyright (C) 2016, Hanno Meyer-Thurow <h.mth@web.de>
@@ -11,6 +11,12 @@
  * Copyright (C) 2018, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
  * Copyright (C) 2019 João Netto <joaonetto901@gmail.com>
  * Copyright (C) 2020 David García Garzón <voki@canvoki.net>
+ * Copyright (C) 2020 Thorsten Behrens <Thorsten.Behrens@CIB.de>
+ * Copyright (C) 2020 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by Technische Universität Dresden
+ * Copyright (C) 2021 Georgiy Sgibnev <georgiy@sgibnev.com>. Work sponsored by lab50.net.
+ * Copyright (C) 2021 Theofilos Intzoglou <int.teo@gmail.com>
+ * Copyright (C) 2022 Alexander Sulfrian <asulfrian@zedat.fu-berlin.de>
+ * Copyright (C) 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,17 +33,27 @@
  * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include "poppler-qt5.h"
+#if defined(MIKTEX_WINDOWS)
+#define MIKTEX_UTF8_WRAP_ALL 1
+#include <miktex/utf8wrap.h>
+#endif
+#include "poppler-form.h"
+
+#include <config.h>
 
 #include <QtCore/QSizeF>
+#include <QUrl>
 
 #include <Form.h>
 #include <Object.h>
 #include <Link.h>
 #include <SignatureInfo.h>
 #include <CertificateInfo.h>
+#include <CryptoSignBackend.h>
+#ifdef ENABLE_NSS3
+#    include <NSSCryptoSignBackend.h>
+#endif
 
-#include "poppler-form.h"
 #include "poppler-page-private.h"
 #include "poppler-private.h"
 #include "poppler-annotation-helper.h"
@@ -45,23 +61,19 @@
 #include <cmath>
 #include <cctype>
 
-#ifdef ENABLE_NSS3
-#    include <hasht.h>
-#endif
-
 namespace {
 
 Qt::Alignment formTextAlignment(::FormWidget *fm)
 {
     Qt::Alignment qtalign = Qt::AlignLeft;
     switch (fm->getField()->getTextQuadding()) {
-    case quaddingCentered:
+    case VariableTextQuadding::centered:
         qtalign = Qt::AlignHCenter;
         break;
-    case quaddingRightJustified:
+    case VariableTextQuadding::rightJustified:
         qtalign = Qt::AlignRight;
         break;
-    case quaddingLeftJustified:
+    case VariableTextQuadding::leftJustified:
         qtalign = Qt::AlignLeft;
     }
     return qtalign;
@@ -111,8 +123,9 @@ FormField::FormField(std::unique_ptr<FormFieldData> dd) : m_formData(std::move(d
         double pageWidth = m_formData->page->getCropWidth();
         double pageHeight = m_formData->page->getCropHeight();
         // landscape and seascape page rotation: be sure to use the correct (== rotated) page size
-        if (((rotation / 90) % 2) == 1)
+        if (((rotation / 90) % 2) == 1) {
             qSwap(pageWidth, pageHeight);
+        }
         for (int i = 0; i < 6; i += 2) {
             MTX[i] = gfxCTM[i] / pageWidth;
             MTX[i + 1] = gfxCTM[i + 1] / pageHeight;
@@ -183,7 +196,14 @@ void FormField::setReadOnly(bool value)
 
 bool FormField::isVisible() const
 {
-    return !(m_formData->fm->getWidgetAnnotation()->getFlags() & Annot::flagHidden);
+    const unsigned int flags = m_formData->fm->getWidgetAnnotation()->getFlags();
+    if (flags & Annot::flagHidden) {
+        return false;
+    }
+    if (flags & Annot::flagNoView) {
+        return false;
+    }
+    return true;
 }
 
 void FormField::setVisible(bool value)
@@ -191,6 +211,7 @@ void FormField::setVisible(bool value)
     unsigned int flags = m_formData->fm->getWidgetAnnotation()->getFlags();
     if (value) {
         flags &= ~Annot::flagHidden;
+        flags &= ~Annot::flagNoView;
     } else {
         flags |= Annot::flagHidden;
     }
@@ -324,15 +345,17 @@ FormFieldIcon FormFieldButton::icon() const
 
 void FormFieldButton::setIcon(const FormFieldIcon &icon)
 {
-    if (FormFieldIconData::getData(icon) == nullptr)
+    if (FormFieldIconData::getData(icon) == nullptr) {
         return;
+    }
 
     FormWidgetButton *fwb = static_cast<FormWidgetButton *>(m_formData->fm);
     if (fwb->getButtonType() == formButtonPush) {
         ::AnnotWidget *w = m_formData->fm->getWidgetAnnotation();
         FormFieldIconData *data = FormFieldIconData::getData(icon);
-        if (data->icon != nullptr)
+        if (data->icon != nullptr) {
             w->setNewAppearance(data->icon->lookup("AP"));
+        }
     }
 }
 
@@ -352,16 +375,18 @@ QList<int> FormFieldButton::siblings() const
 {
     FormWidgetButton *fwb = static_cast<FormWidgetButton *>(m_formData->fm);
     ::FormFieldButton *ffb = static_cast<::FormFieldButton *>(fwb->getField());
-    if (fwb->getButtonType() == formButtonPush)
+    if (fwb->getButtonType() == formButtonPush) {
         return QList<int>();
+    }
 
     QList<int> ret;
     for (int i = 0; i < ffb->getNumSiblings(); ++i) {
         ::FormFieldButton *sibling = static_cast<::FormFieldButton *>(ffb->getSibling(i));
         for (int j = 0; j < sibling->getNumWidgets(); ++j) {
             FormWidget *w = sibling->getWidget(j);
-            if (w)
+            if (w) {
                 ret.append(w->getID());
+            }
         }
     }
 
@@ -380,10 +405,11 @@ FormField::FormType FormFieldText::type() const
 FormFieldText::TextType FormFieldText::textType() const
 {
     FormWidgetText *fwt = static_cast<FormWidgetText *>(m_formData->fm);
-    if (fwt->isFileSelect())
+    if (fwt->isFileSelect()) {
         return FormFieldText::FileSelect;
-    else if (fwt->isMultiline())
+    } else if (fwt->isMultiline()) {
         return FormFieldText::Multiline;
+    }
     return FormFieldText::Normal;
 }
 
@@ -463,8 +489,9 @@ FormFieldChoice::FormType FormFieldChoice::type() const
 FormFieldChoice::ChoiceType FormFieldChoice::choiceType() const
 {
     FormWidgetChoice *fwc = static_cast<FormWidgetChoice *>(m_formData->fm);
-    if (fwc->isCombo())
+    if (fwc->isCombo()) {
         return FormFieldChoice::ComboBox;
+    }
     return FormFieldChoice::ListBox;
 }
 
@@ -512,9 +539,11 @@ QList<int> FormFieldChoice::currentChoices() const
     FormWidgetChoice *fwc = static_cast<FormWidgetChoice *>(m_formData->fm);
     int num = fwc->getNumChoices();
     QList<int> choices;
-    for (int i = 0; i < num; ++i)
-        if (fwc->isSelected(i))
+    for (int i = 0; i < num; ++i) {
+        if (fwc->isSelected(i)) {
             choices.append(i);
+        }
+    }
     return choices;
 }
 
@@ -522,18 +551,20 @@ void FormFieldChoice::setCurrentChoices(const QList<int> &choice)
 {
     FormWidgetChoice *fwc = static_cast<FormWidgetChoice *>(m_formData->fm);
     fwc->deselectAll();
-    for (int i = 0; i < choice.count(); ++i)
+    for (int i = 0; i < choice.count(); ++i) {
         fwc->select(choice.at(i));
+    }
 }
 
 QString FormFieldChoice::editChoice() const
 {
     FormWidgetChoice *fwc = static_cast<FormWidgetChoice *>(m_formData->fm);
 
-    if (fwc->isCombo() && fwc->hasEdit())
+    if (fwc->isCombo() && fwc->hasEdit()) {
         return UnicodeParsedString(fwc->getEditChoice());
-    else
+    } else {
         return QString();
+    }
 }
 
 void FormFieldChoice::setEditChoice(const QString &text)
@@ -571,6 +602,7 @@ public:
 
     EntityInfo issuer_info;
     EntityInfo subject_info;
+    QString nick_name;
     QByteArray certificate_der;
     QByteArray serial_number;
     QByteArray public_key;
@@ -582,8 +614,14 @@ public:
     int version;
     bool is_self_signed;
     bool is_null;
+    CertificateInfo::KeyLocation keyLocation;
 };
 
+CertificateInfo::CertificateInfo() : d_ptr(new CertificateInfoPrivate())
+{
+    d_ptr->is_null = true;
+}
+
 CertificateInfo::CertificateInfo(CertificateInfoPrivate *priv) : d_ptr(priv) { }
 
 CertificateInfo::CertificateInfo(const CertificateInfo &other) : d_ptr(other.d_ptr) { }
@@ -592,8 +630,9 @@ CertificateInfo::~CertificateInfo() = default;
 
 CertificateInfo &CertificateInfo::operator=(const CertificateInfo &other)
 {
-    if (this != &other)
+    if (this != &other) {
         d_ptr = other.d_ptr;
+    }
 
     return *this;
 }
@@ -650,6 +689,12 @@ QString CertificateInfo::subjectInfo(EntityInfoKey key) const
     }
 }
 
+QString CertificateInfo::nickName() const
+{
+    Q_D(const CertificateInfo);
+    return d->nick_name;
+}
+
 QDateTime CertificateInfo::validityStart() const
 {
     Q_D(const CertificateInfo);
@@ -667,26 +712,40 @@ CertificateInfo::KeyUsageExtensions CertificateInfo::keyUsageExtensions() const
     Q_D(const CertificateInfo);
 
     KeyUsageExtensions kuExtensions = KuNone;
-    if (d->ku_extensions & KU_DIGITAL_SIGNATURE)
+    if (d->ku_extensions & KU_DIGITAL_SIGNATURE) {
         kuExtensions |= KuDigitalSignature;
-    if (d->ku_extensions & KU_NON_REPUDIATION)
+    }
+    if (d->ku_extensions & KU_NON_REPUDIATION) {
         kuExtensions |= KuNonRepudiation;
-    if (d->ku_extensions & KU_KEY_ENCIPHERMENT)
+    }
+    if (d->ku_extensions & KU_KEY_ENCIPHERMENT) {
         kuExtensions |= KuKeyEncipherment;
-    if (d->ku_extensions & KU_DATA_ENCIPHERMENT)
+    }
+    if (d->ku_extensions & KU_DATA_ENCIPHERMENT) {
         kuExtensions |= KuDataEncipherment;
-    if (d->ku_extensions & KU_KEY_AGREEMENT)
+    }
+    if (d->ku_extensions & KU_KEY_AGREEMENT) {
         kuExtensions |= KuKeyAgreement;
-    if (d->ku_extensions & KU_KEY_CERT_SIGN)
+    }
+    if (d->ku_extensions & KU_KEY_CERT_SIGN) {
         kuExtensions |= KuKeyCertSign;
-    if (d->ku_extensions & KU_CRL_SIGN)
+    }
+    if (d->ku_extensions & KU_CRL_SIGN) {
         kuExtensions |= KuClrSign;
-    if (d->ku_extensions & KU_ENCIPHER_ONLY)
+    }
+    if (d->ku_extensions & KU_ENCIPHER_ONLY) {
         kuExtensions |= KuEncipherOnly;
+    }
 
     return kuExtensions;
 }
 
+CertificateInfo::KeyLocation CertificateInfo::keyLocation() const
+{
+    Q_D(const CertificateInfo);
+    return d->keyLocation;
+}
+
 QByteArray CertificateInfo::publicKey() const
 {
     Q_D(const CertificateInfo);
@@ -726,10 +785,29 @@ QByteArray CertificateInfo::certificateData() const
     return d->certificate_der;
 }
 
+bool CertificateInfo::checkPassword(const QString &password) const
+{
+#ifdef ENABLE_SIGNATURES
+    auto backend = CryptoSign::Factory::createActive();
+    if (!backend) {
+        return false;
+    }
+    Q_D(const CertificateInfo);
+    auto sigHandler = backend->createSigningHandler(d->nick_name.toStdString(), HashAlgorithm::Sha256);
+    unsigned char buffer[5];
+    memcpy(buffer, "test", 5);
+    sigHandler->addData(buffer, 5);
+    std::optional<GooString> tmpSignature = sigHandler->signDetached(password.toStdString());
+    return tmpSignature.has_value();
+#else
+    return false;
+#endif
+}
+
 class SignatureValidationInfoPrivate
 {
 public:
-    SignatureValidationInfoPrivate(CertificateInfo &&ci) : cert_info(ci) { }
+    explicit SignatureValidationInfoPrivate(CertificateInfo &&ci) : cert_info(ci) { }
 
     SignatureValidationInfo::SignatureStatus signature_status;
     SignatureValidationInfo::CertificateStatus certificate_status;
@@ -740,7 +818,7 @@ public:
     QString signer_subject_dn;
     QString location;
     QString reason;
-    int hash_algorithm;
+    HashAlgorithm hash_algorithm;
     time_t signing_time;
     QList<qint64> range_bounds;
     qint64 docLength;
@@ -790,24 +868,26 @@ QString SignatureValidationInfo::reason() const
 
 SignatureValidationInfo::HashAlgorithm SignatureValidationInfo::hashAlgorithm() const
 {
-#ifdef ENABLE_NSS3
+#ifdef ENABLE_SIGNATURES
     Q_D(const SignatureValidationInfo);
 
     switch (d->hash_algorithm) {
-    case HASH_AlgMD2:
+    case ::HashAlgorithm::Md2:
         return HashAlgorithmMd2;
-    case HASH_AlgMD5:
+    case ::HashAlgorithm::Md5:
         return HashAlgorithmMd5;
-    case HASH_AlgSHA1:
+    case ::HashAlgorithm::Sha1:
         return HashAlgorithmSha1;
-    case HASH_AlgSHA256:
+    case ::HashAlgorithm::Sha256:
         return HashAlgorithmSha256;
-    case HASH_AlgSHA384:
+    case ::HashAlgorithm::Sha384:
         return HashAlgorithmSha384;
-    case HASH_AlgSHA512:
+    case ::HashAlgorithm::Sha512:
         return HashAlgorithmSha512;
-    case HASH_AlgSHA224:
+    case ::HashAlgorithm::Sha224:
         return HashAlgorithmSha224;
+    case ::HashAlgorithm::Unknown:
+        return HashAlgorithmUnknown;
     }
 #endif
     return HashAlgorithmUnknown;
@@ -842,8 +922,9 @@ bool SignatureValidationInfo::signsTotalDocument() const
         // A potential range after d->range_bounds.value(3) would be also not
         // authenticated. Therefore d->range_bounds.value(3) should coincide with
         // the end of the document.
-        if (d->docLength == d->range_bounds.value(3) && !d->signature.isEmpty())
+        if (d->docLength == d->range_bounds.value(3) && !d->signature.isEmpty()) {
             return true;
+        }
     }
     return false;
 }
@@ -856,8 +937,9 @@ CertificateInfo SignatureValidationInfo::certificateInfo() const
 
 SignatureValidationInfo &SignatureValidationInfo::operator=(const SignatureValidationInfo &other)
 {
-    if (this != &other)
+    if (this != &other) {
         d_ptr = other.d_ptr;
+    }
 
     return *this;
 }
@@ -888,6 +970,9 @@ FormFieldSignature::SignatureType FormFieldSignature::signatureType() const
     case unknown_signature_type:
         sigType = UnknownSignatureType;
         break;
+    case unsigned_signature_field:
+        sigType = UnsignedSignature;
+        break;
     }
     return sigType;
 }
@@ -897,19 +982,29 @@ SignatureValidationInfo FormFieldSignature::validate(ValidateOptions opt) const
     return validate(opt, QDateTime());
 }
 
-SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &validationTime) const
+static CertificateInfo::KeyLocation fromPopplerCore(KeyLocation location)
 {
-    FormWidgetSignature *fws = static_cast<FormWidgetSignature *>(m_formData->fm);
-    const time_t validationTimeT = validationTime.isValid() ? validationTime.toTime_t() : -1;
-    SignatureInfo *si = fws->validateSignature(opt & ValidateVerifyCertificate, opt & ValidateForceRevalidation, validationTimeT);
+    switch (location) {
+    case KeyLocation::Computer:
+        return CertificateInfo::KeyLocation::Computer;
+    case KeyLocation::Other:
+        return CertificateInfo::KeyLocation::Other;
+    case KeyLocation::Unknown:
+        return CertificateInfo::KeyLocation::Unknown;
+    case KeyLocation::HardwareToken:
+        return CertificateInfo::KeyLocation::HardwareToken;
+    }
+    return CertificateInfo::KeyLocation::Unknown;
+}
 
-    // get certificate info
-    const X509CertificateInfo *ci = si->getCertificateInfo();
+static CertificateInfoPrivate *createCertificateInfoPrivate(const X509CertificateInfo *ci)
+{
     CertificateInfoPrivate *certPriv = new CertificateInfoPrivate;
     certPriv->is_null = true;
     if (ci) {
         certPriv->version = ci->getVersion();
         certPriv->ku_extensions = ci->getKeyUsageExtensions();
+        certPriv->keyLocation = fromPopplerCore(ci->getKeyLocation());
 
         const GooString &certSerial = ci->getSerialNumber();
         certPriv->serial_number = QByteArray(certSerial.c_str(), certSerial.getLength());
@@ -926,9 +1021,11 @@ SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &v
         certPriv->subject_info.email_address = subjectInfo.email.c_str();
         certPriv->subject_info.org_name = subjectInfo.organization.c_str();
 
+        certPriv->nick_name = ci->getNickName().c_str();
+
         X509CertificateInfo::Validity certValidity = ci->getValidity();
-        certPriv->validity_start = QDateTime::fromTime_t(certValidity.notBefore, Qt::UTC);
-        certPriv->validity_end = QDateTime::fromTime_t(certValidity.notAfter, Qt::UTC);
+        certPriv->validity_start = QDateTime::fromSecsSinceEpoch(certValidity.notBefore, Qt::UTC);
+        certPriv->validity_end = QDateTime::fromSecsSinceEpoch(certValidity.notAfter, Qt::UTC);
 
         const X509CertificateInfo::PublicKeyInfo &pkInfo = ci->getPublicKeyInfo();
         certPriv->public_key = QByteArray(pkInfo.publicKey.c_str(), pkInfo.publicKey.getLength());
@@ -941,6 +1038,19 @@ SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &v
         certPriv->is_null = false;
     }
 
+    return certPriv;
+}
+
+SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &validationTime) const
+{
+    FormWidgetSignature *fws = static_cast<FormWidgetSignature *>(m_formData->fm);
+    const time_t validationTimeT = validationTime.isValid() ? validationTime.toSecsSinceEpoch() : -1;
+    SignatureInfo *si = fws->validateSignature(opt & ValidateVerifyCertificate, opt & ValidateForceRevalidation, validationTimeT, !(opt & ValidateWithoutOCSPRevocationCheck), opt & ValidateUseAIACertFetch);
+
+    // get certificate info
+    const X509CertificateInfo *ci = si->getCertificateInfo();
+    CertificateInfoPrivate *certPriv = createCertificateInfoPrivate(ci);
+
     SignatureValidationInfoPrivate *priv = new SignatureValidationInfoPrivate(CertificateInfo(certPriv));
     switch (si->getSignatureValStatus()) {
     case SIGNATURE_VALID:
@@ -990,11 +1100,11 @@ SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &v
         priv->certificate_status = SignatureValidationInfo::CertificateNotVerified;
         break;
     }
-    priv->signer_name = si->getSignerName();
-    priv->signer_subject_dn = si->getSubjectDN();
+    priv->signer_name = QString::fromStdString(si->getSignerName());
+    priv->signer_subject_dn = QString::fromStdString(si->getSubjectDN());
     priv->hash_algorithm = si->getHashAlgorithm();
-    priv->location = si->getLocation();
-    priv->reason = si->getReason();
+    priv->location = UnicodeParsedString(si->getLocation().toStr());
+    priv->reason = UnicodeParsedString(si->getReason().toStr());
 
     priv->signing_time = si->getSigningTime();
     const std::vector<Goffset> ranges = fws->getSignedRangeBounds();
@@ -1003,13 +1113,191 @@ SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &v
             priv->range_bounds.append(bound);
         }
     }
-    GooString *checkedSignature = fws->getCheckedSignature(&priv->docLength);
+    const std::optional<GooString> checkedSignature = fws->getCheckedSignature(&priv->docLength);
     if (priv->range_bounds.size() == 4 && checkedSignature) {
         priv->signature = QByteArray::fromHex(checkedSignature->c_str());
     }
-    delete checkedSignature;
 
     return SignatureValidationInfo(priv);
 }
 
+FormFieldSignature::SigningResult FormFieldSignature::sign(const QString &outputFileName, const PDFConverter::NewSignatureData &data) const
+{
+    FormWidgetSignature *fws = static_cast<FormWidgetSignature *>(m_formData->fm);
+    if (fws->signatureType() != unsigned_signature_field) {
+        return FieldAlreadySigned;
+    }
+
+    Goffset file_size = 0;
+    const std::optional<GooString> sig = fws->getCheckedSignature(&file_size);
+    if (sig) {
+        // the above unsigned_signature_field check
+        // should already catch this, but double check
+        return FieldAlreadySigned;
+    }
+    const auto reason = std::unique_ptr<GooString>(data.reason().isEmpty() ? nullptr : QStringToUnicodeGooString(data.reason()));
+    const auto location = std::unique_ptr<GooString>(data.location().isEmpty() ? nullptr : QStringToUnicodeGooString(data.location()));
+    const auto ownerPwd = std::optional<GooString>(data.documentOwnerPassword().constData());
+    const auto userPwd = std::optional<GooString>(data.documentUserPassword().constData());
+    const auto gSignatureText = std::unique_ptr<GooString>(QStringToUnicodeGooString(data.signatureText()));
+    const auto gSignatureLeftText = std::unique_ptr<GooString>(QStringToUnicodeGooString(data.signatureLeftText()));
+
+    const bool success = fws->signDocumentWithAppearance(outputFileName.toStdString(), data.certNickname().toStdString(), data.password().toStdString(), reason.get(), location.get(), ownerPwd, userPwd, *gSignatureText, *gSignatureLeftText,
+                                                         data.fontSize(), data.leftFontSize(), convertQColor(data.fontColor()), data.borderWidth(), convertQColor(data.borderColor()), convertQColor(data.backgroundColor()));
+
+    return success ? SigningSuccess : GenericSigningError;
+}
+
+bool hasNSSSupport()
+{
+#ifdef ENABLE_NSS3
+    return true;
+#else
+    return false;
+#endif
+}
+
+QVector<CertificateInfo> getAvailableSigningCertificates()
+{
+    auto backend = CryptoSign::Factory::createActive();
+    if (!backend) {
+        return {};
+    }
+    QVector<CertificateInfo> vReturnCerts;
+    std::vector<std::unique_ptr<X509CertificateInfo>> vCerts = backend->getAvailableSigningCertificates();
+
+    for (auto &cert : vCerts) {
+        CertificateInfoPrivate *certPriv = createCertificateInfoPrivate(cert.get());
+        vReturnCerts.append(CertificateInfo(certPriv));
+    }
+
+    return vReturnCerts;
+}
+
+static std::optional<CryptoSignBackend> convertToFrontend(std::optional<CryptoSign::Backend::Type> type)
+{
+    if (!type) {
+        return std::nullopt;
+    }
+    switch (type.value()) {
+    case CryptoSign::Backend::Type::NSS3:
+        return CryptoSignBackend::NSS;
+    case CryptoSign::Backend::Type::GPGME:
+        return CryptoSignBackend::GPG;
+    }
+    return std::nullopt;
+}
+
+static std::optional<CryptoSign::Backend::Type> convertToBackend(std::optional<CryptoSignBackend> backend)
+{
+    if (!backend) {
+        return std::nullopt;
+    }
+    switch (backend.value()) {
+    case CryptoSignBackend::NSS:
+        return CryptoSign::Backend::Type::NSS3;
+    case CryptoSignBackend::GPG:
+        return CryptoSign::Backend::Type::GPGME;
+    }
+    return std::nullopt;
+}
+
+QVector<CryptoSignBackend> availableCryptoSignBackends()
+{
+    QVector<CryptoSignBackend> backends;
+    for (auto &backend : CryptoSign::Factory::getAvailable()) {
+        auto converted = convertToFrontend(backend);
+        if (converted) {
+            backends.push_back(converted.value());
+        }
+    }
+    return backends;
+}
+
+std::optional<CryptoSignBackend> activeCryptoSignBackend()
+{
+    return convertToFrontend(CryptoSign::Factory::getActive());
+}
+
+bool setActiveCryptoSignBackend(CryptoSignBackend backend)
+{
+    auto available = availableCryptoSignBackends();
+    if (!available.contains(backend)) {
+        return false;
+    }
+    auto converted = convertToBackend(backend);
+    if (!converted) {
+        return false;
+    }
+    CryptoSign::Factory::setPreferredBackend(converted.value());
+    return activeCryptoSignBackend() == backend;
+}
+
+static bool hasNSSBackendFeature(CryptoSignBackendFeature feature)
+{
+    switch (feature) {
+    case CryptoSignBackendFeature::BackendAsksPassphrase:
+        return false;
+    }
+    return false;
+}
+
+static bool hasGPGBackendFeature(CryptoSignBackendFeature feature)
+{
+    switch (feature) {
+    case CryptoSignBackendFeature::BackendAsksPassphrase:
+        return true;
+    }
+    return false;
+}
+
+bool hasCryptoSignBackendFeature(CryptoSignBackend backend, CryptoSignBackendFeature feature)
+{
+    switch (backend) {
+    case CryptoSignBackend::NSS:
+        return hasNSSBackendFeature(feature);
+    case CryptoSignBackend::GPG:
+        return hasGPGBackendFeature(feature);
+    }
+    return false;
+}
+
+QString POPPLER_QT5_EXPORT getNSSDir()
+{
+#ifdef ENABLE_NSS3
+    return QString::fromLocal8Bit(NSSSignatureConfiguration::getNSSDir().c_str());
+#else
+    return QString();
+#endif
+}
+
+void setNSSDir(const QString &path)
+{
+#ifdef ENABLE_NSS3
+    if (path.isEmpty()) {
+        return;
+    }
+
+    GooString *goo = QStringToGooString(path);
+    NSSSignatureConfiguration::setNSSDir(*goo);
+    delete goo;
+#else
+    (void)path;
+#endif
+}
+
+namespace {
+std::function<QString(const QString &)> nssPasswordCall;
+}
+
+void setNSSPasswordCallback(const std::function<char *(const char *)> &f)
+{
+#ifdef ENABLE_NSS3
+    NSSSignatureConfiguration::setNSSPasswordCallback(f);
+#else
+    qWarning() << "setNSSPasswordCallback called but this poppler is built without NSS support";
+    (void)f;
+#endif
+}
+
 }
