@@ -7,17 +7,22 @@
 // Copyright (C) 2011-2015, 2017 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2012 Arseny Solokha <asolokha@gmx.com>
 // Copyright (C) 2012 Fabio D'Urso <fabiodurso@hotmail.it>
-// Copyright (C) 2012, 2014, 2017-2019 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2012, 2014, 2017-2019, 2021, 2022 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2013 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2013 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2015 Arthur Stavisky <vovodroid@gmail.com>
 // Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2019 Marek Kasik <mkasik@redhat.com>
-// Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2019, 2023 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2022 crt <chluo@cse.cuhk.edu.hk>
 //
 //========================================================================
 
+#if defined(MIKTEX_WINDOWS)
+#define MIKTEX_UTF8_WRAP_ALL 1
+#include <miktex/utf8wrap.h>
+#endif
 #include <PDFDoc.h>
 #include <GlobalParams.h>
 #include "parseargs.h"
@@ -48,7 +53,7 @@ static void doMergeNameTree(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldR
                     if (mkey.isString() && mvalue.isRef()) {
                         if (mkey.getString()->cmp(key.getString()) < 0) {
                             newNameArray->add(Object(new GooString(mkey.getString()->c_str())));
-                            newNameArray->add(Object({ mvalue.getRef().num + numOffset, mvalue.getRef().gen }));
+                            newNameArray->add(Object(Ref { mvalue.getRef().num + numOffset, mvalue.getRef().gen }));
                             j += 2;
                         } else if (mkey.getString()->cmp(key.getString()) == 0) {
                             j += 2;
@@ -68,7 +73,7 @@ static void doMergeNameTree(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldR
             const Object &mvalue = mergeNameArray.arrayGetNF(j + 1);
             if (mkey.isString() && mvalue.isRef()) {
                 newNameArray->add(Object(new GooString(mkey.getString()->c_str())));
-                newNameArray->add(Object({ mvalue.getRef().num + numOffset, mvalue.getRef().gen }));
+                newNameArray->add(Object(Ref { mvalue.getRef().num + numOffset, mvalue.getRef().gen }));
             }
             j += 2;
         }
@@ -81,7 +86,7 @@ static void doMergeNameTree(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldR
             const Object &value = mergeNameArray.arrayGetNF(i + 1);
             if (key.isString() && value.isRef()) {
                 newNameArray->add(Object(new GooString(key.getString()->c_str())));
-                newNameArray->add(Object({ value.getRef().num + numOffset, value.getRef().gen }));
+                newNameArray->add(Object(Ref { value.getRef().num + numOffset, value.getRef().gen }));
             }
         }
         srcNameTree->add("Names", Object(newNameArray));
@@ -105,16 +110,21 @@ static void doMergeNameDict(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldR
     }
 }
 
-static void doMergeFormDict(Dict *srcFormDict, Dict *mergeFormDict, int numOffset)
+static bool doMergeFormDict(Dict *srcFormDict, Dict *mergeFormDict, int numOffset)
 {
     Object srcFields = srcFormDict->lookup("Fields");
     Object mergeFields = mergeFormDict->lookup("Fields");
     if (srcFields.isArray() && mergeFields.isArray()) {
         for (int i = 0; i < mergeFields.arrayGetLength(); i++) {
             const Object &value = mergeFields.arrayGetNF(i);
-            srcFields.arrayAdd(Object({ value.getRef().num + numOffset, value.getRef().gen }));
+            if (!value.isRef()) {
+                error(errSyntaxError, -1, "Fields object is not a Ref.");
+                return false;
+            }
+            srcFields.arrayAdd(Object(Ref { value.getRef().num + numOffset, value.getRef().gen }));
         }
     }
+    return true;
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -137,13 +147,11 @@ int main(int argc, char *argv[])
     OutStream *outStr;
     int i;
     int j, rootNum;
-    std::vector<PDFDoc *> docs;
+    std::vector<std::unique_ptr<PDFDoc>> docs;
     int majorVersion = 0;
     int minorVersion = 0;
     char *fileName = argv[argc - 1];
-    int exitCode;
 
-    exitCode = 99;
     const bool ok = parseArgs(argDesc, &argc, argv);
     if (!ok || argc < 3 || printVersion || printHelp) {
         fprintf(stderr, "pdfunite version %s\n", PACKAGE_VERSION);
@@ -152,18 +160,16 @@ int main(int argc, char *argv[])
         if (!printVersion) {
             printUsage("pdfunite", "<PDF-sourcefile-1>..<PDF-sourcefile-n> <PDF-destfile>", argDesc);
         }
-        if (printVersion || printHelp)
-            exitCode = 0;
-        return exitCode;
+        if (printVersion || printHelp) {
+            return 0;
+        }
+        return 99;
     }
-    exitCode = 0;
     globalParams = std::make_unique<GlobalParams>();
 
     for (i = 1; i < argc - 1; i++) {
-        GooString *gfileName = new GooString(argv[i]);
-        PDFDoc *doc = new PDFDoc(gfileName, nullptr, nullptr, nullptr);
+        std::unique_ptr<PDFDoc> doc = std::make_unique<PDFDoc>(std::make_unique<GooString>(argv[i]));
         if (doc->isOk() && !doc->isEncrypted() && doc->getXRef()->getCatalog().isDict()) {
-            docs.push_back(doc);
             if (doc->getPDFMajorVersion() > majorVersion) {
                 majorVersion = doc->getPDFMajorVersion();
                 minorVersion = doc->getPDFMinorVersion();
@@ -172,6 +178,7 @@ int main(int argc, char *argv[])
                     minorVersion = doc->getPDFMinorVersion();
                 }
             }
+            docs.push_back(std::move(doc));
         } else if (doc->isOk()) {
             if (doc->isEncrypted()) {
                 error(errUnimplemented, -1, "Could not merge encrypted files ('{0:s}')", argv[i]);
@@ -204,6 +211,14 @@ int main(int argc, char *argv[])
     Object ocObj;
     if (docs.size() >= 1) {
         Object catObj = docs[0]->getXRef()->getCatalog();
+        if (!catObj.isDict()) {
+            fclose(f);
+            delete yRef;
+            delete countRef;
+            delete outStr;
+            error(errSyntaxError, -1, "XRef's Catalog is not a dictionary.");
+            return -1;
+        }
         Dict *catDict = catObj.getDict();
         intents = catDict->lookup("OutputIntents");
         afObj = catDict->lookupNF("AcroForm").copy();
@@ -282,9 +297,17 @@ int main(int argc, char *argv[])
             }
 
             const PDFRectangle *cropBox = nullptr;
-            if (docs[i]->getCatalog()->getPage(j)->isCropped())
+            if (docs[i]->getCatalog()->getPage(j)->isCropped()) {
                 cropBox = docs[i]->getCatalog()->getPage(j)->getCropBox();
-            docs[i]->replacePageDict(j, docs[i]->getCatalog()->getPage(j)->getRotate(), docs[i]->getCatalog()->getPage(j)->getMediaBox(), cropBox);
+            }
+            if (!docs[i]->replacePageDict(j, docs[i]->getCatalog()->getPage(j)->getRotate(), docs[i]->getCatalog()->getPage(j)->getMediaBox(), cropBox)) {
+                fclose(f);
+                delete yRef;
+                delete countRef;
+                delete outStr;
+                error(errSyntaxError, -1, "PDFDoc::replacePageDict failed.");
+                return -1;
+            }
             Ref *refPage = docs[i]->getCatalog()->getPageRef(j);
             Object page = docs[i]->getXRef()->fetch(*refPage);
             Dict *pageDict = page.getDict();
@@ -301,20 +324,34 @@ int main(int argc, char *argv[])
             }
         }
         Object pageCatObj = docs[i]->getXRef()->getCatalog();
+        if (!pageCatObj.isDict()) {
+            fclose(f);
+            delete yRef;
+            delete countRef;
+            delete outStr;
+            error(errSyntaxError, -1, "XRef's Catalog is not a dictionary.");
+            return -1;
+        }
         Dict *pageCatDict = pageCatObj.getDict();
         Object pageNames = pageCatDict->lookup("Names");
         if (!pageNames.isNull() && pageNames.isDict()) {
             if (!names.isDict()) {
                 names = Object(new Dict(yRef));
             }
-            doMergeNameDict(docs[i], yRef, countRef, 0, 0, names.getDict(), pageNames.getDict(), numOffset);
+            doMergeNameDict(docs[i].get(), yRef, countRef, 0, 0, names.getDict(), pageNames.getDict(), numOffset);
         }
         Object pageForm = pageCatDict->lookup("AcroForm");
         if (i > 0 && !pageForm.isNull() && pageForm.isDict()) {
             if (afObj.isNull()) {
                 afObj = pageCatDict->lookupNF("AcroForm").copy();
             } else if (afObj.isDict()) {
-                doMergeFormDict(afObj.getDict(), pageForm.getDict(), numOffset);
+                if (!doMergeFormDict(afObj.getDict(), pageForm.getDict(), numOffset)) {
+                    fclose(f);
+                    delete yRef;
+                    delete countRef;
+                    delete outStr;
+                    return -1;
+                }
             }
         }
         objectsCount += docs[i]->writePageObjects(outStr, yRef, numOffset, true);
@@ -357,8 +394,9 @@ int main(int argc, char *argv[])
     yRef->add(rootNum + 1, 0, outStr->getPos(), true);
     outStr->printf("%d 0 obj\n", rootNum + 1);
     outStr->printf("<< /Type /Pages /Kids [");
-    for (j = 0; j < (int)pages.size(); j++)
+    for (j = 0; j < (int)pages.size(); j++) {
         outStr->printf(" %d 0 R", rootNum + j + 2);
+    }
     outStr->printf(" ] /Count %zd >>\nendobj\n", pages.size());
     objectsCount++;
 
@@ -368,8 +406,9 @@ int main(int argc, char *argv[])
         outStr->printf("<< ");
         Dict *pageDict = pages[i].getDict();
         for (j = 0; j < pageDict->getLength(); j++) {
-            if (j > 0)
+            if (j > 0) {
                 outStr->printf(" ");
+            }
             const char *key = pageDict->getKey(j);
             Object value = pageDict->getValNF(j).copy();
             if (strcmp(key, "Parent") == 0) {
@@ -396,7 +435,5 @@ int main(int argc, char *argv[])
     fclose(f);
     delete yRef;
     delete countRef;
-    for (i = 0; i < (int)docs.size(); i++)
-        delete docs[i];
-    return exitCode;
+    return 0;
 }
