@@ -1,6 +1,6 @@
 /* LinuxOsRelease.cpp: 
 
-   Copyright (C) 2018-2020 Christian Schenk
+   Copyright (C) 2018-2021 Christian Schenk
 
    This file is part of the MiKTeX Core Library.
 
@@ -24,7 +24,7 @@
 #include <iomanip>
 
 #include <miktex/Core/Cfg>
-#include <miktex/Core/PathName>
+#include <miktex/Util/PathName>
 #include <miktex/Core/tux/LinuxOsRelease>
 
 #include "internal.h"
@@ -32,6 +32,7 @@
 using namespace std;
 
 using namespace MiKTeX::Core;
+using namespace MiKTeX::Util;
 
 string Unquote(const string& s)
 {
