@@ -2,7 +2,7 @@
  * @file miktex/KPSE/Emulation.h
  * @defgroup KPSE Kpathsea emulation
  * @author Christian Schenk
- * @brief  Utilities for emulating the Kpathsea library
+ * @brief Utilities for emulating the Kpathsea library
  *
  * @copyright Copyright © 2000-2024 Christian Schenk
  *
@@ -152,26 +152,6 @@ MIKTEX_END_EXTERN_C_BLOCK;
 
 /// @}
 
-/* _________________________________________________________________________
- *
- * c-memstr.h
- *
- */
-
-/// @name c-memstr.h
-/// Stuff from `c-memstr.h`.
-/// @{
-
-#if !defined(HAVE_INDEX) && !defined(index)
-#define index(s, c) strchr(s, c)
-#endif
-
-#if !defined(HAVE_RINDEX) && !defined(rindex)
-#define rindex(s, c) strrchr(s, c)
-#endif
-
-/// @}
-
 /* _________________________________________________________________________
  *
  * c-pathch.h
