@@ -3,7 +3,7 @@
  * @author Christian Schenk
  * @brief PackageManager implementation
  *
- * @copyright Copyright © 2001-2022 Christian Schenk
+ * @copyright Copyright © 2001-2024 Christian Schenk
  *
  * This file is part of MiKTeX Package Manager.
  *
@@ -100,7 +100,7 @@ void PackageManagerImpl::Lock(chrono::milliseconds timeout)
 {
     if (lockFile == nullptr)
     {
-        lockFile = LockFile::Create(session->GetSpecialPath(SpecialPath::InstallRoot) / PathName(MIKTEX_PATH_PACKAGE_MANAGER_LOCK));
+        lockFile = LockFile::Create(session->GetSpecialPath(SpecialPath::InstallRoot) / MIKTEX_PATH_PACKAGE_MANAGER_LOCK);
     }
     if (!lockFile->TryLock(timeout))
     {
@@ -156,7 +156,7 @@ void PackageManagerImpl::LoadDatabase(const PathName& path, bool isArchive)
         unique_ptr<MiKTeX::Extractor::Extractor> extractor(MiKTeX::Extractor::Extractor::CreateExtractor(DB_ARCHIVE_FILE_TYPE));
         extractor->Extract(absPath, tempDir->GetPathName());
 
-        packageManifestsPath = tempDir->GetPathName() / PathName(MIKTEX_PACKAGE_MANIFESTS_INI_FILENAME);
+        packageManifestsPath = tempDir->GetPathName() / MIKTEX_PACKAGE_MANIFESTS_INI_FILENAME;
     }
     else
     {
@@ -533,8 +533,8 @@ bool PackageManager::IsLocalPackageRepository(const PathName& path)
     }
 
     // local mirror of remote package repository?
-    PathName file1 = PathName(path, PathName(MIKTEX_REPOSITORY_MANIFEST_ARCHIVE_FILE_NAME));
-    PathName file2 = PathName(path, PathName(MIKTEX_PACKAGE_MANIFESTS_ARCHIVE_FILE_NAME));
+    PathName file1 = PathName(path / MIKTEX_REPOSITORY_MANIFEST_ARCHIVE_FILE_NAME);
+    PathName file2 = PathName(path / MIKTEX_PACKAGE_MANIFESTS_ARCHIVE_FILE_NAME);
     if (File::Exists(file1) && File::Exists(file2))
     {
         return true;
@@ -969,7 +969,7 @@ PackageInfo PackageManager::GetPackageManifest(const Cfg& cfg, const string& pac
 #if defined(MIKTEX_UNIX)
                 path.ConvertToUnix();
 #endif
-                if (texmfPrefix.empty() || (PathName::Compare(PathName(texmfPrefix), path, texmfPrefix.length()) == 0))
+                if (texmfPrefix.empty() || (PathName::ComparePrefixes(PathName(texmfPrefix), path, texmfPrefix.length()) == 0))
                 {
                     packageInfo.runFiles.push_back(path.ToString());
                 }
@@ -987,7 +987,7 @@ PackageInfo PackageManager::GetPackageManifest(const Cfg& cfg, const string& pac
 #if defined(MIKTEX_UNIX)
                 path.ConvertToUnix();
 #endif
-                if (texmfPrefix.empty() || (PathName::Compare(PathName(texmfPrefix), path, texmfPrefix.length()) == 0))
+                if (texmfPrefix.empty() || (PathName::ComparePrefixes(PathName(texmfPrefix), path, texmfPrefix.length()) == 0))
                 {
                     packageInfo.docFiles.push_back(path.ToString());
                 }
@@ -1005,7 +1005,7 @@ PackageInfo PackageManager::GetPackageManifest(const Cfg& cfg, const string& pac
 #if defined(MIKTEX_UNIX)
                 path.ConvertToUnix();
 #endif
-                if (texmfPrefix.empty() || (PathName::Compare(PathName(texmfPrefix), path, texmfPrefix.length()) == 0))
+                if (texmfPrefix.empty() || (PathName::ComparePrefixes(PathName(texmfPrefix), path, texmfPrefix.length()) == 0))
                 {
                     packageInfo.sourceFiles.push_back(path.ToString());
                 }
