@@ -2,7 +2,7 @@
  * Gregorio is a program that translates gabc files to GregorioTeX
  * This file provides functions for determining glyphs from notes.
  *
- * Copyright (C) 2006-2021 The Gregorio Project (see CONTRIBUTORS.md)
+ * Copyright (C) 2006-2025 The Gregorio Project (see CONTRIBUTORS.md)
  *
  * This file is part of Gregorio.
  *
@@ -138,10 +138,13 @@ static char add_note_to_a_glyph(gregorio_glyph_type current_glyph_type,
         break;
     case S_FLAT:
     case S_FLAT_PAREN:
+    case S_FLAT_SOFT:
     case S_SHARP:
     case S_SHARP_PAREN:
+    case S_SHARP_SOFT:
     case S_NATURAL:
     case S_NATURAL_PAREN:
+    case S_NATURAL_SOFT:
         next_glyph_type = G_ALTERATION;
         *end_of_glyph = DET_END_OF_BOTH;
         break;
@@ -1097,6 +1100,7 @@ gregorio_glyph *gabc_det_glyphs_from_notes(gregorio_note *current_note,
                     case S_QUILISMA:
                     case S_QUADRATUM:
                     case S_QUILISMA_QUADRATUM:
+                    case S_VIRGA:
                         /* these are fusible */
                         if (current_glyph_type <= G_PUNCTA_INCLINATA
                                 || current_note->u.note.shape != S_PUNCTUM) {
@@ -1289,11 +1293,15 @@ gregorio_glyph *gabc_det_glyphs_from_notes(gregorio_note *current_note,
             }
         }
 
-        if (!next_note && current_glyph_type != G_UNDETERMINED) {
+        if ((!next_note && current_glyph_type != G_UNDETERMINED)
+                || (autofuse && current_note
+                    && current_note->u.note.shape == S_VIRGA)) {
             /* we must end the determination here */
             current_note = close_glyph(&last_glyph, current_glyph_type,
                     &current_glyph_first_note, liquescentia, current_note,
                     punctum_inclinatum_orientation);
+            next_glyph_type = current_glyph_type = G_UNDETERMINED;
+            liquescentia = L_NO_LIQUESCENTIA;
         }
 
         last_pitch = current_note->u.note.pitch;
