@@ -2,7 +2,7 @@
 ** DVIToSVG.cpp                                                         **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2023 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2025 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -54,13 +54,8 @@
 #include "HtmlSpecialHandler.hpp"
 #include "PapersizeSpecialHandler.hpp"
 #include "PdfSpecialHandler.hpp"
+#include "PsSpecialHandlerProxy.hpp"
 #include "TpicSpecialHandler.hpp"
-#ifndef HAVE_LIBGS
-	#include "NoPsSpecialHandler.hpp"
-#endif
-#ifndef DISABLE_GS
-	#include "PsSpecialHandler.hpp"
-#endif
 
 ///////////////////////////////////
 
@@ -107,7 +102,7 @@ void DVIToSVG::convert (unsigned first, unsigned last, HashFunction *hashFunc) {
 			hashFunc->update(PAGE_HASH_SETTINGS.optionsHash());
 			combinedHash = hashFunc->digestString();
 		}
-		const SVGOutput::HashTriple hashTriple(dviHash, shortenedOptHash, combinedHash);
+		const SVGOutput::HashTriple hashTriple(dviHash, shortenedOptHash, std::move(combinedHash));
 		FilePath path = _out.filepath(i, numberOfPages(), hashTriple);
 		if (!dviHash.empty() && !PAGE_HASH_SETTINGS.isSet(HashSettings::P_REPLACE) && path.exists()) {
 			Message::mstream(false, Message::MC_PAGE_NUMBER) << "skipping page " << i;
@@ -124,10 +119,17 @@ void DVIToSVG::convert (unsigned first, unsigned last, HashFunction *hashFunc) {
 			string fname = path.shorterAbsoluteOrRelative();
 			if (fname.empty())
 				fname = "<stdout>";
-			if (success)
-				Message::mstream(false, Message::MC_PAGE_WRITTEN) << "\noutput written to " << fname << '\n';
-			else
+			if (!success)
 				Message::wstream(true) << "failed to write output to " << fname << '\n';
+			else {
+				Message::mstream(false, Message::MC_PAGE_WRITTEN) << "\noutput written to " << fname << '\n';
+				if (!_userMessage.empty()) {
+					if (auto specialActions = dynamic_cast<SpecialActions*>(_actions.get())) {
+						string msg = specialActions->expandText(_userMessage);
+						Message::ustream(true) << msg << "\n";
+					}
+				}
+			}
 			_svg.reset();
 			_actions->reset();
 		}
@@ -148,7 +150,7 @@ static unique_ptr<HashFunction> create_hash_function (const string &algo) {
 		msg += name + ", ";
 	msg.pop_back();
 	msg.back() = ')';
-	throw MessageException(msg);
+	throw MessageException(std::move(msg));
 }
 
 
@@ -298,13 +300,12 @@ void DVIToSVG::leaveEndPage (unsigned) {
 		Message::wstream(false) << "\npage is empty\n";
 	if (_bboxFormatString != "none") {
 		_svg.setBBox(bbox);
-		const double bp2pt = 72.27/72;
-		const double bp2mm = 25.4/72;
+		const double bp2pt = (1_bp).pt();
 		Message::mstream(false) << '\n';
 		Message::mstream(false, Message::MC_PAGE_SIZE) << "graphic size: " << XMLString(bbox.width()*bp2pt) << "pt"
 			" x " << XMLString(bbox.height()*bp2pt) << "pt"
-			" (" << XMLString(bbox.width()*bp2mm) << "mm"
-			" x " << XMLString(bbox.height()*bp2mm) << "mm)";
+			" (" << XMLString(bbox.width()) << "bp"
+			" x " << XMLString(bbox.height()) << "bp)";
 		Message::mstream(false) << '\n';
 	}
 }
@@ -416,28 +417,9 @@ void DVIToSVG::setProcessSpecials (const char *ignorelist, bool pswarning) {
 		SpecialManager::registerHandler<EmSpecialHandler>(ignoredHandlerName);         // handles emTeX specials
 		SpecialManager::registerHandler<HtmlSpecialHandler>(ignoredHandlerName);       // handles hyperref specials
 		SpecialManager::registerHandler<PapersizeSpecialHandler>(ignoredHandlerName);  // handles papersize special
-		SpecialManager::registerHandler<PdfSpecialHandler>(ignoredHandlerName);        // handles pdf specials
+		SpecialManager::registerHandler<PdfSpecialHandler>(ignoredHandlerName);
 		SpecialManager::registerHandler<TpicSpecialHandler>(ignoredHandlerName);       // handles tpic specials
-		if (find(ignoredHandlerName.begin(), ignoredHandlerName.end(), PsSpecialHandler::handlerName()) == ignoredHandlerName.end()) {
-#ifndef DISABLE_GS
-			if (Ghostscript().available())
-				SpecialManager::registerHandler<PsSpecialHandler>(ignoredHandlerName);   // handles PostScript specials
-			else
-#endif
-			{
-#ifndef HAVE_LIBGS
-				// dummy PS special handler that only prints warning messages
-				SpecialManager::registerHandler<NoPsSpecialHandler>(ignoredHandlerName);
-				if (pswarning) {
-#ifdef DISABLE_GS
-					Message::wstream() << "processing of PostScript specials has been disabled permanently\n";
-#else
-					Message::wstream() << "processing of PostScript specials is disabled (Ghostscript not found)\n";
-#endif
-				}
-#endif
-			}
-		}
+		SpecialManager::registerHandler(util::make_unique<PsSpecialHandlerProxy>(pswarning), ignoredHandlerName);
 	}
 }
 
@@ -597,7 +579,7 @@ void DVIToSVG::HashSettings::setParameters (const string &paramstr) {
 				msg.pop_back();
 				msg.pop_back();
 				msg += ')';
-				throw MessageException(msg);
+				throw MessageException(std::move(msg));
 			}
 		}
 	}
