@@ -2,7 +2,7 @@
 ** TriangularPatch.hpp                                                  **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2022 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2025 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -33,7 +33,6 @@ class TriangularPatch : public ShadingPatch {
 		int psShadingType() const override {return 4;}
 		DPair valueAt (double u, double v) const;
 		Color colorAt (double u, double v) const;
-		Color averageColor() const override;
 		void setPoints (const PointVec &points, int edgeflag, ShadingPatch *patch) override;
 		void setPoints (const DPair &p1, const DPair &p2, const DPair &p3);
 		void setColors (const ColorVec &colors, int edgeflag, ShadingPatch *patch) override;
@@ -44,9 +43,6 @@ class TriangularPatch : public ShadingPatch {
 		int numPoints (int edgeflag) const override {return edgeflag == 0 ? 3 : 1;}
 		int numColors (int edgeflag) const override {return edgeflag == 0 ? 3 : 1;}
 
-	protected:
-		Color averageColor (const Color &c1, const Color &c2, const Color &c3) const;
-
 	private:
 		DPair _points[3];
 		Color _colors[3];
