@@ -8,9 +8,9 @@
 //and partial result progress to support streaming of results.
 struct  ProgressParams
 {
-	std::pair<boost::optional<std::string> , boost::optional<int> > token;
-	lsp::Any value;
-	MAKE_SWAP_METHOD(ProgressParams, token, value)
+        std::pair<optional<std::string> , optional<int> > token;
+        lsp::Any value;
+        MAKE_SWAP_METHOD(ProgressParams, token, value)
 };
 MAKE_REFLECT_STRUCT(ProgressParams, token, value)
 DEFINE_NOTIFICATION_TYPE(Notify_Progress, ProgressParams, "$/progress");
