@@ -60,7 +60,7 @@ private:
   bool isCustom;
 
 private:
-  std::shared_ptr<MiKTeX::Core::Session> session = MiKTeX::Core::Session::Get();
+  std::shared_ptr<MiKTeX::Core::Session> session = MIKTEX_SESSION();
 };
 
 #endif
