@@ -68,7 +68,7 @@ public:
   void Reload();
 
 private:
-  std::shared_ptr<MiKTeX::Core::Session> session = MiKTeX::Core::Session::Get();
+  std::shared_ptr<MiKTeX::Core::Session> session = MIKTEX_SESSION();
 
 private:
   struct InternalLanguageInfo :
