@@ -1,6 +1,6 @@
 /* runbat.cpp: the MiKTeX batch file runner
 
-   Copyright (C) 2004-2019 Christian Schenk
+   Copyright (C) 2004-2021 Christian Schenk
 
    This file is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
@@ -21,6 +21,10 @@
 #include <miktex/Core/Exceptions>
 #include <miktex/Util/StringUtil>
 
+#if defined(MIKTEX_WINDOWS)
+#include <miktex/Core/win/ConsoleCodePageSwitcher>
+#endif
+
 #include <string>
 #include <vector>
 
@@ -36,6 +40,9 @@ int wmain(int argc, wchar_t** argv)
 int main(int argc, char** argv)
 #endif
 {
+#if defined(MIKTEX_WINDOWS)
+  ConsoleCodePageSwitcher cpSwitcher;
+#endif
   string programName = "runbat";
   Application app;
   try
@@ -65,13 +72,13 @@ int main(int argc, char** argv)
   }
   catch (const MiKTeXException& e)
   {
-    Application::Sorry(programName, e);
+    app.Sorry(programName, e);
     e.Save();
     return 1;
   }
   catch (const std::exception& e)
   {
-    Application::Sorry(programName, e);
+    app.Sorry(programName, e);
     return 1;
   }
 }
