@@ -152,14 +152,6 @@ const struct poptOption IniTeXMFApp::options[] = {
     nullptr
   },
 
-  {
-    "log-file", 0,
-    POPT_ARG_STRING | POPT_ARGFLAG_DOC_HIDDEN, nullptr,
-    OPT_LOG_FILE,
-    nullptr,
-    nullptr
-  },
-
   {
     "mklangs", 0,
     POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN, nullptr,
