@@ -68,7 +68,7 @@ static const char *const callbacknames[] = {
     "finish_pdffile",
     "finish_pdfpage",
     "pre_dump","start_file", "stop_file",
-    "show_error_message","show_lua_error_hook",
+    "show_error_message","show_lua_error_hook","show_ignored_error_message",
     "show_warning_message",
     "hpack_quality", "vpack_quality",
     "process_rule",
@@ -85,6 +85,7 @@ static const char *const callbacknames[] = {
     "make_extensible",
     "process_pdf_image_content",
     "provide_charproc_data",
+    "input_level_string",
     NULL
 };
 
