@@ -525,7 +525,8 @@ void begin_token_list(halfword p, quarterword t)
             iloc = token_link(p);
             if (tracing_macros_par > 1) {
                 begin_diagnostic();
-                tprint_nl("");
+             // tprint_nl("");
+                print_input_level();
                 if (t == mark_text)
                     tprint_esc("mark");
                 else if (t == write_text)
@@ -661,7 +662,7 @@ void begin_file_reading(void)
         check_buffer_overflow(first);
     incr(in_open);
     push_input();
-    iindex = (unsigned char) in_open;
+    iindex = (unsigned short)(in_open);
     source_filename_stack[iindex] = 0;
     full_source_filename_stack[iindex] = NULL;
     eof_seen[iindex] = false;
@@ -696,6 +697,11 @@ void end_file_reading(void)
     } else if (iname > 17) {
         /*tex Forget it. */
         lua_a_close_in(cur_file, 0);
+        source_filename_stack[iindex] = 0;
+        if (full_source_filename_stack[iindex] != NULL) {
+            free(full_source_filename_stack[iindex]);
+            full_source_filename_stack[iindex] = NULL;
+        }
     }
     pop_input();
     decr(in_open);
@@ -743,6 +749,7 @@ void initialize_inputstack(void)
         buffer[first] = 0;
         decr(first);
     } while (first != 0);
+    buffer[0] = 0;
     scanner_status = normal;
     warning_index = null;
     first = 1;
