@@ -1084,6 +1084,18 @@ static boolean math_and_text_reversed_p(void)
 
 */
 
+static int only_dirs(halfword n)
+{
+    while (n) {
+        if (type(n) == local_par_node || type(n) == dir_node) {
+            n = vlink(n);
+        } else {
+            return 0;
+        }
+    }
+    return 1;
+}
+
 void enter_display_math(void)
 {
     /*tex new or partial |pre_display_size| */
@@ -1101,19 +1113,21 @@ void enter_display_math(void)
         \.{\$\${ }\$\$}
 
     */
-    if (head == tail ||
-        (vlink(head) == tail &&
-         type(tail) == local_par_node && vlink(tail) == null)) {
-        if (vlink(head) == tail) {
-            /*tex
+    if (head == tail) {
+        pop_nest();
+        w = -max_dimen;
+    } else if ((vlink(head) == tail &&
+               type(tail) == local_par_node && vlink(tail) == null) ||
+               (math_empty_display_mode_par == 1 && only_dirs(vlink(head)))) {
+        /*tex
 
-                |resume_after_display| inserts a |local_par_node|, but if there
-                is another display immediately following, we have to get rid of
-                that node.
+            We ignore |null| paragraphs, that is those that only have a local par node,
+            for example because |resume_after_display| inserts a |local_par_node|.
+            If |\mathdisplayemptymode=1| paragraphs caontaining only local par node
+            and possibly a few dir nodes are ignored as well.
 
-            */
-            flush_node(tail);
-        }
+        */
+        flush_node_list(vlink(head));
         pop_nest();
         w = -max_dimen;
     } else {
@@ -1462,6 +1476,8 @@ int scan_math(pointer p, int mstyle)
     math_character(p) = mval.character_value;
     if ((mval.class_value == math_use_current_family_code) && cur_fam_par_in_range)
         math_fam(p) = cur_fam_par;
+    else if ((mval.family_value == var_fam_par) && var_fam_par_in_range)
+        math_fam(p) = cur_fam_par;
     else
         math_fam(p) = mval.family_value;
     return 0;
@@ -1508,6 +1524,9 @@ void set_math_char(mathcodeval mval)
             if (cur_fam_par_in_range)
                 math_fam(nucleus(p)) = cur_fam_par;
             subtype(p) = ord_noad_type;
+        } else if ((mval.family_value == var_fam_par) && var_fam_par_in_range) {
+            if (cur_fam_par_in_range)
+                math_fam(nucleus(p)) = cur_fam_par;
         } else {
             math_class_to_type(subtype(p),mval.class_value);
         }
@@ -1802,6 +1821,8 @@ void math_ac(void)
         math_character(top_accent_chr(tail)) = t.character_value;
         if ((t.class_value == math_use_current_family_code) && cur_fam_par_in_range)
             math_fam(top_accent_chr(tail)) = cur_fam_par;
+        else if ((t.family_value == var_fam_par) && var_fam_par_in_range)
+            math_fam(top_accent_chr(tail)) = cur_fam_par;
         else
             math_fam(top_accent_chr(tail)) = t.family_value;
     }
@@ -1811,6 +1832,8 @@ void math_ac(void)
         math_character(bot_accent_chr(tail)) = b.character_value;
         if ((b.class_value == math_use_current_family_code) && cur_fam_par_in_range)
             math_fam(bot_accent_chr(tail)) = cur_fam_par;
+        else if ((b.family_value == var_fam_par) && var_fam_par_in_range)
+            math_fam(bot_accent_chr(tail)) = cur_fam_par;
         else
             math_fam(bot_accent_chr(tail)) = b.family_value;
     }
@@ -1820,6 +1843,8 @@ void math_ac(void)
         math_character(overlay_accent_chr(tail)) = o.character_value;
         if ((o.class_value == math_use_current_family_code) && cur_fam_par_in_range)
             math_fam(overlay_accent_chr(tail)) = cur_fam_par;
+        else if ((o.family_value == var_fam_par) && var_fam_par_in_range)
+            math_fam(overlay_accent_chr(tail)) = cur_fam_par;
         else
             math_fam(overlay_accent_chr(tail)) = o.family_value;
     }
@@ -2098,12 +2123,15 @@ void close_math_group(pointer p)
     if (p != null && vlink(p) == null) {
         if (type(p) == simple_noad) {
             if (subscr(p) == null && supscr(p) == null) {
-                /*tex (subtype(p) == ord_noad_type) */
+                /*tex 
+                    In traditional \TEX\ this only happens for ordinary noads.
+                */
+             // int flatten = subtype(p) == ord_noad_type; /*tex traditional \TEX. */
                 int flatten = 0;
                 int modepar = math_flatten_mode_par;
                 switch (subtype(p)) {
                     case ord_noad_type :
-                        flatten = (modepar & 1) == 1;
+                        flatten = (modepar & 1) == 1; /* the default */
                         break;
                     case bin_noad_type :
                         flatten = (modepar & 2) == 2;
@@ -2499,7 +2527,7 @@ static void finish_displayed_math(boolean l, pointer eqno_box, pointer p)
                 if (type(p) == glue_node)
                     d = 0;
     }
-    tail_append(new_penalty(pre_display_penalty_par,after_display_penalty));
+    tail_append(new_penalty(pre_display_penalty_par,before_display_penalty));
 
     /* tex
        By default the short skip detection is not adapted to r2l typesetting and that
@@ -2517,7 +2545,10 @@ static void finish_displayed_math(boolean l, pointer eqno_box, pointer p)
         g2 = below_display_short_skip_code;
        }
     } else {
-      if ((d + line_s <= pre_display_size_par) || ((! dir_math_save && l) || (dir_math_save && ! l))) {
+      boolean math_and_par_reversed = math_and_text_reversed_p();
+      if ((d + line_s <= pre_display_size_par) || (eqno_box != null
+              && ((! math_and_par_reversed && l)
+              || (math_and_par_reversed && ! l)))) {
         /*tex not enough clearance */
         g1 = above_display_skip_code;
         g2 = below_display_skip_code;
@@ -2550,53 +2581,33 @@ static void finish_displayed_math(boolean l, pointer eqno_box, pointer p)
     if (eqno_w != 0) {
         r = new_kern(line_w - eq_w - eqno_w - d);
         if (l) {
-            if (swap_dir) {
-                if (math_direction_par==dir_TLT) {
-                    /*tex TRT + TLT + \eqno: (swap_dir=true,  math_direction_par=TLT, l=true) */
-                    s = new_kern(width(r) + eqno_w);
-                    try_couple_nodes(eqno_box,r);
-                    try_couple_nodes(r,eq_box);
-                    try_couple_nodes(eq_box,s);
-                } else {
-                    /*tex TLT + TRT + \eqno: (swap_dir=true,  math_direction_par=TRT, l=true) */
-                    try_couple_nodes(eqno_box,r);
-                    try_couple_nodes(r,eq_box);
-                }
+            if (math_direction_par==dir_TLT) {
+                /*tex TRT + TLT + \eqno: (swap_dir=true, math_direction_par=TLT, l=true) */
+                /*tex TLT + TLT + \leqno: (swap_dir=false, math_direction_par=TLT, l=true) */
+                s = new_kern(width(r) + eqno_w);
             } else {
-                if (math_direction_par==dir_TLT) {
-                    /*tex TLT + TLT + \leqno: (swap_dir=false, math_direction_par=TLT, l=true) */
-                    s = new_kern(width(r) + eqno_w);
-                } else {
-                    /*tex TRT + TRT + \leqno: (swap_dir=false, math_direction_par=TRT, l=true) */
-                    s = new_kern(width(r));
-                }
-                try_couple_nodes(eqno_box,r);
-                try_couple_nodes(r,eq_box);
-                try_couple_nodes(eq_box,s);
+                /*tex TLT + TRT + \eqno: (swap_dir=true, math_direction_par=TRT, l=true) */
+                /*tex TRT + TRT + \leqno: (swap_dir=false, math_direction_par=TRT, l=true) */
+                s = new_kern(d);
             }
+            try_couple_nodes(eqno_box,r);
+            try_couple_nodes(r,eq_box);
+            try_couple_nodes(eq_box,s);
             eq_box = eqno_box;
         } else {
-            if (swap_dir) {
-                if (math_direction_par==dir_TLT) {
-                    /*tex TRT + TLT + \leqno: (swap_dir=true,  math_direction_par=TLT, l=false) */
-                } else {
-                    /*tex TLT + TRT + \leqno: (swap_dir=true,  math_direction_par=TRT, l=false) */
-                }
-                try_couple_nodes(eq_box,r);
-                try_couple_nodes(r,eqno_box);
+            if (math_direction_par==dir_TLT) {
+                /*tex TRT + TLT + \leqno: (swap_dir=true,  math_direction_par=TLT, l=false) */
+                /*tex TLT + TLT + \eqno: (swap_dir=false, math_direction_par=TLT, l=false) */
+                s = new_kern(d);
             } else {
-                if (math_direction_par==dir_TLT) {
-                    /*tex TLT + TLT + \eqno: (swap_dir=false, math_direction_par=TLT, l=false) */
-                    s = new_kern(d);
-                } else {
-                    /*tex TRT + TRT + \eqno: (swap_dir=false, math_direction_par=TRT, l=false) */
-                    s = new_kern(width(r) + eqno_w);
-                }
-                try_couple_nodes(s,eq_box);
-                try_couple_nodes(eq_box,r);
-                try_couple_nodes(r,eqno_box);
-                eq_box = s;
+                /*tex TLT + TRT + \leqno: (swap_dir=true,  math_direction_par=TRT, l=false) */
+                /*tex TRT + TRT + \eqno: (swap_dir=false, math_direction_par=TRT, l=false) */
+                s = new_kern(width(r) + eqno_w);
             }
+            try_couple_nodes(s,eq_box);
+            try_couple_nodes(eq_box,r);
+            try_couple_nodes(r,eqno_box);
+            eq_box = s;
         }
         eq_box = hpack(eq_box, 0, additional, -1);
         subtype(eq_box) = equation_list; /* new */
