@@ -22,25 +22,25 @@
 #define KCAT_KANJI_IVS 23
 
 /* TOKEN */
-boolean check_kanji (integer c)
+integer check_kanji (integer c)
 {
     integer c0, c1;
 
-    if (c >= CS_TOKEN_FLAG) return false;
+    if (c >= CS_TOKEN_FLAG) return 0;
 
     c0 = c & CJK_TOKEN_FLAG;
     c1 = XXHi(c);
     if (c1>=CAT_LEFT_BRACE && c1<=CAT_DELIM_NUM &&
             c0 < LATIN_CHAR_LIMIT) { /* kcatcode latin_ucs */
-        return is_char_kanji(c0);
+        return 1;
     }
     else if (c1>=KCAT_KANJI && c1<=KCAT_MODIFIER) {
-        return is_char_kanji(c0);
+        return is_char_kanji(c0)*2;
     }
     else if (c1>=KCAT_KANJI_IVS+1 && c1<=KCAT_KANJI_IVS+4) {
-        return is_char_kanji(c - KCAT_KANJI_IVS * CJK_CHAR_LIMIT);
+        return is_char_kanji(c - KCAT_KANJI_IVS * CJK_CHAR_LIMIT)*3;
     }
-    return false;
+    return 0;
 }
 
 boolean is_char_ascii(integer c)
@@ -97,8 +97,8 @@ integer ktoken_to_chr(integer c)
 }
 
 /* Ref. http://www.unicode.org/Public/UNIDATA/Blocks.txt */
-/* # Blocks-16.0.0.txt                                   */
-/* # Date: 2024-02-22                                    */
+/* # Blocks-17.0.0.txt                                   */
+/* # Date: 2025-08-01                                    */
 static long ucs_range[]={
       0x0000, /* Basic Latin					     */ /* 0x00 */
       0x0080, /* Latin-1 Supplement				     */
@@ -295,6 +295,7 @@ static long ucs_range[]={
       0x108E0, /* Hatran					     */ /* 0xc0 */
       0x10900, /* Phoenician					     */
       0x10920, /* Lydian					     */
+      0x10940, /* Sidetic					     */
       0x10980, /* Meroitic Hieroglyphs				     */
       0x109A0, /* Meroitic Cursive				     */
       0x10A00, /* Kharoshthi					     */
@@ -307,8 +308,8 @@ static long ucs_range[]={
       0x10B80, /* Psalter Pahlavi				     */
       0x10C00, /* Old Turkic					     */
       0x10C80, /* Old Hungarian				     */
-      0x10D00, /* Hanifi Rohingya				     */
-      0x10D40, /* Garay					     */ /* 0xd0 */
+      0x10D00, /* Hanifi Rohingya				     */ /* 0xd0 */
+      0x10D40, /* Garay					     */
       0x10E60, /* Rumi Numeral Symbols				     */
       0x10E80, /* Yezidi					     */
       0x10EC0, /* Arabic Extended-C				     */
@@ -323,8 +324,8 @@ static long ucs_range[]={
       0x11100, /* Chakma					     */
       0x11150, /* Mahajani					     */
       0x11180, /* Sharada					     */
-      0x111E0, /* Sinhala Archaic Numbers			     */
-      0x11200, /* Khojki					     */ /* 0xe0 */
+      0x111E0, /* Sinhala Archaic Numbers			     */ /* 0xe0 */
+      0x11200, /* Khojki					     */
       0x11280, /* Multani					     */
       0x112B0, /* Khudawadi					     */
       0x11300, /* Grantha					     */
@@ -339,24 +340,26 @@ static long ucs_range[]={
       0x11700, /* Ahom						     */
       0x11800, /* Dogra					     */
       0x118A0, /* Warang Citi					     */
-      0x11900, /* Dives Akuru					     */
-      0x119A0, /* Nandinagari					     */ /* 0xf0 */
+      0x11900, /* Dives Akuru					     */ /* 0xf0 */
+      0x119A0, /* Nandinagari					     */
       0x11A00, /* Zanabazar Square				     */
       0x11A50, /* Soyombo					     */
       0x11AB0, /* Unified Canadian Aboriginal Syllabics Extended-A   */
       0x11AC0, /* Pau Cin Hau					     */
       0x11B00, /* Devanagari Extended-A			     */
+      0x11B60, /* Sharada Supplement				     */
       0x11BC0, /* Sunuwar					     */
       0x11C00, /* Bhaiksuki					     */
       0x11C70, /* Marchen					     */
       0x11D00, /* Masaram Gondi				     */
       0x11D60, /* Gunjala Gondi				     */
+      0x11DB0, /* Tolong Siki					     */
       0x11EE0, /* Makasar					     */
       0x11F00, /* Kawi						     */
-      0x11FB0, /* Lisu Supplement				     */
+      0x11FB0, /* Lisu Supplement				     */ /* 0x100 */
       0x11FC0, /* Tamil Supplement				     */
       0x12000, /* Cuneiform					     */
-      0x12400, /* Cuneiform Numbers and Punctuation		     */ /* 0x100 */
+      0x12400, /* Cuneiform Numbers and Punctuation		     */
       0x12480, /* Early Dynastic Cuneiform			     */
       0x12F90, /* Cypro-Minoan					     */
       0x13000, /* Egyptian Hieroglyphs				     */
@@ -369,26 +372,29 @@ static long ucs_range[]={
       0x16A70, /* Tangsa					     */
       0x16AD0, /* Bassa Vah					     */
       0x16B00, /* Pahawh Hmong					     */
-      0x16D40, /* Kirat Rai					     */
+      0x16D40, /* Kirat Rai					     */ /* 0x110 */
       0x16E40, /* Medefaidrin					     */
+      0x16EA0, /* Beria Erfe					     */
       0x16F00, /* Miao						     */
-      0x16FE0, /* Ideographic Symbols and Punctuation		     */ /* 0x110 */
+      0x16FE0, /* Ideographic Symbols and Punctuation		     */
       0x17000, /* Tangut					     */
       0x18800, /* Tangut Components				     */
       0x18B00, /* Khitan Small Script				     */
       0x18D00, /* Tangut Supplement				     */
+      0x18D80, /* Tangut Components Supplement			     */
       0x1AFF0, /* Kana Extended-B				     */
       0x1B000, /* Kana Supplement				     */
       0x1B100, /* Kana Extended-A				     */
       0x1B130, /* Small Kana Extension				     */
       0x1B170, /* Nushu					     */
       0x1BC00, /* Duployan					     */
-      0x1BCA0, /* Shorthand Format Controls			     */
+      0x1BCA0, /* Shorthand Format Controls			     */ /* 0x120 */
       0x1CC00, /* Symbols for Legacy Computing Supplement	     */
+      0x1CEC0, /* Miscellaneous Symbols Supplement		     */
       0x1CF00, /* Znamenny Musical Notation			     */
       0x1D000, /* Byzantine Musical Symbols			     */
       0x1D100, /* Musical Symbols				     */
-      0x1D200, /* Ancient Greek Musical Notation		     */ /* 0x120 */
+      0x1D200, /* Ancient Greek Musical Notation		     */
       0x1D2C0, /* Kaktovik Numerals				     */
       0x1D2E0, /* Mayan Numerals				     */
       0x1D300, /* Tai Xuan Jing Symbols			     */
@@ -398,13 +404,14 @@ static long ucs_range[]={
       0x1DF00, /* Latin Extended-G				     */
       0x1E000, /* Glagolitic Supplement			     */
       0x1E030, /* Cyrillic Extended-D				     */
-      0x1E100, /* Nyiakeng Puachue Hmong			     */
+      0x1E100, /* Nyiakeng Puachue Hmong			     */ /* 0x130 */
       0x1E290, /* Toto						     */
       0x1E2C0, /* Wancho					     */
       0x1E4D0, /* Nag Mundari					     */
       0x1E5D0, /* Ol Onal					     */
+      0x1E6C0, /* Tai Yo					     */
       0x1E7E0, /* Ethiopic Extended-B				     */
-      0x1E800, /* Mende Kikakui				     */ /* 0x130 */
+      0x1E800, /* Mende Kikakui				     */
       0x1E900, /* Adlam					     */
       0x1EC70, /* Indic Siyaq Numbers				     */
       0x1ED00, /* Ottoman Siyaq Numbers			     */
@@ -413,14 +420,14 @@ static long ucs_range[]={
       0x1F030, /* Domino Tiles					     */
       0x1F0A0, /* Playing Cards				     */
       0x1F100, /* Enclosed Alphanumeric Supplement		     */
-      0x1F200, /* Enclosed Ideographic Supplement		     */
+      0x1F200, /* Enclosed Ideographic Supplement		     */ /* 0x140 */
       0x1F300, /* Miscellaneous Symbols and Pictographs	     */
       0x1F600, /* Emoticons					     */
       0x1F650, /* Ornamental Dingbats				     */
       0x1F680, /* Transport and Map Symbols			     */
       0x1F700, /* Alchemical Symbols				     */
       0x1F780, /* Geometric Shapes Extended			     */
-      0x1F800, /* Supplemental Arrows-C			     */ /* 0x140 */
+      0x1F800, /* Supplemental Arrows-C			     */
       0x1F900, /* Supplemental Symbols and Pictographs		     */
       0x1FA00, /* Chess Symbols				     */
       0x1FA70, /* Symbols and Pictographs Extended-A		     */
@@ -429,14 +436,14 @@ static long ucs_range[]={
       0x2A700, /* CJK Unified Ideographs Extension C		     */
       0x2B740, /* CJK Unified Ideographs Extension D		     */
       0x2B820, /* CJK Unified Ideographs Extension E		     */
-      0x2CEB0, /* CJK Unified Ideographs Extension F		     */
+      0x2CEB0, /* CJK Unified Ideographs Extension F		     */ /* 0x150 */
       0x2EBF0, /* CJK Unified Ideographs Extension I		     */
       0x2F800, /* CJK Compatibility Ideographs Supplement	     */
       0x30000, /* CJK Unified Ideographs Extension G		     */
       0x31350, /* CJK Unified Ideographs Extension H		     */
-      0x323B0, /* CJK Unified Ideographs Extension J (Unicode 17.0)  */
+      0x323B0, /* CJK Unified Ideographs Extension J		     */
       0x33480, /* reserved					     */
-      0x40000, /* reserved					     */ /* 0x150 */
+      0x40000, /* reserved					     */
       0x50000, /* reserved					     */
       0x60000, /* reserved					     */
       0x70000, /* reserved					     */
@@ -445,7 +452,7 @@ static long ucs_range[]={
       0xA0000, /* reserved					     */
       0xB0000, /* reserved					     */
       0xC0000, /* reserved					     */
-      0xD0000, /* reserved					     */
+      0xD0000, /* reserved					     */ /* 0x160 */
       0xE0000, /* Tags						     */
       0xE0100, /* Variation Selectors Supplement		     */
       0xE01F0, /* reserved					     */
@@ -454,7 +461,7 @@ static long ucs_range[]={
   /* Value over 0x10FFFF is illegal under Unicode,
      We use for some special application.  */
       0x110000, /* Reserved					     */
-      0x120000, /* Reserved					     */ /* 0x160 */
+      0x120000, /* Reserved					     */
       0x130000, /* Reserved					     */
       0x140000, /* Reserved					     */
       0x150000, /* Reserved					     */
@@ -463,14 +470,14 @@ static long ucs_range[]={
       0x180000, /* Reserved					     */
       0x190000, /* Reserved					     */
       0x1A0000, /* Reserved					     */
-      0x1B0000, /* Reserved					     */
+      0x1B0000, /* Reserved					     */ /* 0x170 */
       0x1C0000, /* Reserved					     */
       0x1D0000, /* Reserved					     */
       0x1E0000, /* Reserved					     */
       0x1F0000, /* Reserved					     */
       0x200000, /* Reserved					     */
       0x210000, /* Reserved					     */
-      0x220000, /* Kana with Voiced Sound Mark			     */ /* 0x170 */
+      0x220000, /* Kana with Voiced Sound Mark			     */
       0x240000, /* Kana with Semi-Voiced Sound Mark		     */
       0x25E6E6, /* Emoji Flag Sequence				     */
       0x260000, /* Emoji with Modifier Fitzpatrick		     */
@@ -478,7 +485,7 @@ static long ucs_range[]={
       0x400000, /* Standardized Variation Sequence		     */
       0x800000, /* Emoji Keycap Sequence			     */
       0x800080, /* Ideographic Variation Sequence, VS17..VS48	     */
-      CJK_CHAR_LIMIT, /* Ideographic Variation Sequence, VS49..VS256 */ /* 0x178 */
+      CJK_CHAR_LIMIT, /* Ideographic Variation Sequence, VS49..VS256 */ /* 0x17F */
       IVS_CHAR_LIMIT
 };
 
@@ -561,11 +568,11 @@ integer kcatcodekey(integer c)
             if (   COMBINING_ENCLOSING_KEYCAP==c  )
             return 0x1FA;
             break;
-        case 0x138:        /* Block : Enclosed Alphanumeric Supplement */
+        case 0x13f:        /* Block : Enclosed Alphanumeric Supplement */
             if (   REGIONAL_INDICATOR_SYMBOL_LETTER_A <=c && c<= REGIONAL_INDICATOR_SYMBOL_LETTER_Z  )
             return 0x1FB;
             break;
-        case 0x13a:        /* Block : Miscellaneous Symbols and Pictographs */
+        case 0x141:        /* Block : Miscellaneous Symbols and Pictographs */
             if (   EMOJI_MODIFIER_FITZPATRIC_TYPE1_2 <=c && c<= EMOJI_MODIFIER_FITZPATRIC_TYPE6  )
             return 0x1FC;
             break;
